package it.unitn.disi.ronchet.esame22_06_27.accessManagement;

import it.unitn.disi.ronchet.esame22_06_27.functionalities.Setup;
import it.unitn.disi.ronchet.esame22_06_27.models.UserProperties;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.annotation.*;
import java.io.IOException;
import java.util.HashMap;

@WebServlet(name = "FrontController", value = "/frontController")
public class FrontController extends HttpServlet {
    ServletContext ctx=null;
    HashMap<String, UserProperties> userList;
    @Override
    public void init(){
        ctx=this.getServletContext();
        //make sure data structure is in place
        if (ctx.getAttribute("userList")==null) new Setup(ctx);
        userList=(HashMap<String, UserProperties>)ctx.getAttribute("userList");
    }
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        System.out.println("===> Entering FrontController doGet");
        //GET requests are redirected to the login page if no session is available
        HttpSession session=request.getSession(false);
        if (session!=null && session.getAttribute("username")!=null) {
            System.out.println("Validate access - session is valid - passing to POST");
            doPost(request, response);
            return;
        }
        System.out.println("Validate access - session is not valid - redirecting to login");
        request.getRequestDispatcher("WEB-INF/login.jsp").forward(request, response);
    }
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        System.out.println("===> Entering FrontController doPost");
        // the request might come from the login page
        String username = null;
        // first, try to see if this request comes from a login
        username = request.getParameter("username");
        if (username != null) {
            boolean success = manageAccessFromLogin(username, request, response);
            if (!success) {
                request.getRequestDispatcher("WEB-INF/login.jsp").forward(request, response);
                return;
            }
        } else {
            // username from request was null, so let's try with the session
            username = manageAccessFromSession(request, response);
            if (username == null) {
                System.out.println("NULL user in session");
                request.getRequestDispatcher("WEB-INF/login.jsp").forward(request, response);
                return;
            }
        }
        String target = "userPageServlet";
        if (username.equals("admin"))
            target = "adminPageServlet";
        System.out.println(username+" => "+target);
        log((username+" => "+target));
        request.getRequestDispatcher(target).forward(request, response);
    }

    /**
     *
     * @param request
     * @param response
     * @return username if ok, null otherwise
     */
    private String manageAccessFromSession(HttpServletRequest request, HttpServletResponse response) {
        System.out.println("===> Entering FrontController manageAccess");
        HttpSession session=request.getSession(false);
        if (session==null) return null; //no session exists
        String username=(String)session.getAttribute("username");
        System.out.println("USER FOUND IN SESSION :"+username);
        return username;
    }

    /**
     *
     * @param username
     * @param request
     * @param response
     * @return true is validation is ok, false otherwise
     */
    private boolean manageAccessFromLogin(String username, HttpServletRequest request, HttpServletResponse response) {
        System.out.println("===> Entering FrontController manageAccessFromLogin");
        if (userList.containsKey(username)) {
            // username is ok, if we already had a session we invalidate it
            HttpSession session=request.getSession(false);
            if (session!=null) session.invalidate();
            System.out.println("VALID USER FOUND IN REQUEST: "+username);
            session=request.getSession(true); // create a new session
            // update user data
            UserProperties up=userList.get(username);
            up.setConnected(true);
            // update info in ctx
            synchronized (ctx) {
                userList.put(username, up);
            }
            session.setAttribute("username",username);
            return true;
        }
        System.out.println("INVALID USER FOUND IN REQUEST: "+username);
        // username is not ok, let's prepare error message
        request.setAttribute("message", " User " + username + " does not exist");
        return false;
    }

}
